@@ -36,12 +36,7 @@ sub import {
 
     init_class($caller);
 
-    if (Any::Moose::moose_is_preferred()) {
-        Moose->import({ into_level => 1 });
-    }
-    else {
-        Mouse->export_to_level( 1 );
-    }
+    any_moose()->import({into_level => 1});
 }
 
 # fix up Interface.
@@ -132,19 +127,10 @@ sub _construct_writer {
         $writer->meta->add_around_method_modifier( $around => $args->{around}->{$around} );
     }
     for my $attribute (keys %{ $args->{attributes} || {} }) {
-        if (Any::Moose::moose_is_preferred()) {
-            $writer->meta->add_attribute( 
-                $attribute,
-                %{ $args->{attributes}->{$attribute} }
-            )
-        }
-        else {
-            Mouse::Meta::Attribute->create( 
-                $writer->meta, 
-                $attribute,
-                %{ $args->{attributes}->{$attribute} } 
-            );
-        }
+        $writer->meta->add_attribute( 
+            $attribute,
+            %{ $args->{attributes}->{$attribute} }
+        )
     }
 
     # FIXME
